#!/usr/bin/python3
# -*- coding: utf-8 -*-

# ~ Calcul des coordonnées du Soleil depuis Jean MEEUS :
# ~ Calculs astronomiques à l'usage des amateurs
# ~ chapitre 15, page 55 et suivantes
#   et d'après Astronmical Algorithms
#   chapitres 24 et 25  

import numpy as np
from math import *
  

# ~ ====================== fonctions =========================

class Soleil2000():
    """
        Pour équinoxe de référence en l'an 2000
        D'après Jean MEEUS Astronmical Algorithms
        chapitres 24 et 25    
    """
    def __init__(self):
        self.jourDeReference = 2451545.0         # le 1.5 janvier 2000

    def longitudeMoyenne(self, T):
        """
        The true geometric longitude ⊙ referred to the mean equinox of the date.
        Formule 24.2 page 151.
        longitude moyenne est la longitude écliptique à laquelle un objet se trouverait 
        si son orbite était circulaire — c'est-à-dire d'excentricité nulle — 
        et d'inclinaison nulle. 
        """
        L = 280.46645 + 36000.76983 * T + 0.0003032 * T**2
        return L     
    
    def longitudeVraie(self, L, C):
        """
        la longitude vraie est la longitude écliptique à laquelle 
        se trouverait un objet si l'inclinaison de son orbite était nulle. 
        """
        return L + C

    def anomalieMoyenne(self, T):
        """
        Formule 24.3 page 151.
        """
        M = 357.52910 + 35999.05030 * T - 0.0001559 * T**2 \
        - 0.00000048 * T**3  # (24.3)
        return M
        
    def excentricite(self, T):
        e = 0.016708617 - 0.000042037 * T - 0.0000001236 * T**2 # (24.4)
        return e
        
    def equationDuCentre(self, T, M):
        C = (1.914600 - 0.004817 * T - 0.000014 * T**2) * sin(radians(M)) \
            + (0.019993 - 0.000101 * T) * sin( radians(2 * M)) \
            + 0.000290 * sin(radians(3 * M))
        return C
     
    def rayonVecteur(self, e, v):
        R = (1.000001018 * (1 - e**2)) / (1 + e * cos(radians(v)))
        return R
        
    def meanObliquityOfEcliptic(self, nb, T):
        """
        Formule 21.2 page 135
        """
        mob = (nb * 3600) - 46.8150 * T - 0.00059 * T**2 + 0.001813 * T**3
        return mob / 3600
        
    
    def longitudeApparente(self, T, L):
        """
            The Sun's Longitude ⊙, obtenue par la méthode décrite plus haut,
            is the true geometric longitude referred to the mean equinox of the date.
            If the apparent Longitude delta of the Sun, referred to the true 
            equinox of the date, is required, it is necessary to correct ⊙ 
            for the nutation and the aberration.
            Unless high acuracy is required, this can be performed as follow.        
        """
        Ω = 125.04 - ( 1934.136 * T )
        # ~ print(Ω)
        delta = L - 0.00569 - ( 0.00478 * sin(radians(Ω)))
        return delta
        
    def coordonneesRect(self, R, rond, mob):
        """
        Calcul des coordonnées rectangulaires équatoriales géocentriques
        Formule (16-1) de "Calculs astronomiques à l'usage des amateurs" 
        page 59
        """
        X = R * cos(radians(rond))
        Y = R * sin(radians(rond)) * cos(radians(mob))
        Z = R * sin(radians(rond)) * sin(radians(mob))
        
        return X , Y, Z
        


