#!/usr/bin/python3
# -*- coding: utf-8 -*-

# ~ Calcul des coordonnées du Soleil depuis Jean MEEUS :
# ~ Calculs astronomiques à l'usage des amateurs
# ~ chapitre 15, page 55 et suivantes
#   et d'après Astronmical Algorithms
#   chapitres 24 et 25  

import numpy as np
from math import *
import soleil

# ~ ====================== fonctions =========================

def sexToDecimal(ang, hparam):
    "Fonction qui convertit '08:03:10.429' en nombre flottant"
    # ~ Paramètres :
        # ~ ang     : chaîne de la forme '08:03:10.429'
        # ~ hparam  : valeur True ou False
    # ~ Retourne l'équivalent en nombre flottant
    # Auteur : Alexander Davenport
    # https://github.com/davenporta/OrbitalDetermination35
    
    anglist = ang.split(":")
    for i in range(len(anglist)):
        anglist[i] = float(anglist[i])
    decimal = (anglist[1] / 60) + (anglist[2] / 3600)
    if anglist[0] < 0:
        ang = anglist[0] - decimal
    else:
        ang = anglist[0] + decimal
    if hparam == True:
        ang *= 15
    return ang
    
def getJo(date):
    "Convertit la date du jour en numéro de jour julien"
    # ~ Paramètres : 
        # ~ date : sous la forme ANNEE-MOIS-JOUR.décimale
    # ~ Retourne le numéro du jour julien
    # Auteur : Alexander Davenport
    # https://github.com/davenporta/OrbitalDetermination35
        
    joArr = date.split("-")
    for i in range(len(joArr)):
        joArr[i] = float(joArr[i])
    jodate = 367*joArr[0] - int((7*(joArr[0] + int((joArr[1]+9)/12)))/4) \
     + int(275 * joArr[1] / 9) + joArr[2] + 1721013.5
    return jodate

def getJD(date, time):
    "Convertit H, MN, SEC en fraction de jour et ajoute le numéro du jour julien"
    # ~ Paramètres : 
        # ~ date : numéro du jour julien
        # ~ time : heure sous la forme '08:03:10.429' H:MN:SEC
    # ~ Retourne le julien complété de la valeur H:MN:SEC  
    # Auteur : Alexander Davenport
    # https://github.com/davenporta/OrbitalDetermination35
    
    UT = sexToDecimal(time, False)
    return date + (UT / 24)
    
def dansLimite360Degres(nb):
    """
    Ramène la valeur de l'angle dans la limite 0 à 360°
    en tenant compte du signe du nombre
    """
    fois = int(abs(nb) // 360)
    if nb > 0:
        return nb - (fois * 360)
    else:
        return nb + ((fois +1 )* 360)
    
    
# ~ ====================== fonctions =========================
# ~ date            = '1992-10-13'
date = '1978-11-12'
heure           = '00:00:00.000'
jourJulien = getJD(getJo(date), heure)
j = soleil.Soleil2000()
print("Test d'une classe Python de calcul d'éléments du Soleil")
print("-------------------------------------------------------")
print("Jour de référence équinoxe 2000     : ", j.jourDeReference)
print("Date                                : ", date)
print("Heure                               : ", heure)
print("Jour Julien pour la date et l'heure : ",jourJulien)
T = (jourJulien - j.jourDeReference) / 36525     # (24.1)
print("Temps en siècle julien        (T)   : ",T)
L = j.longitudeMoyenne(T)
print("Longitude moyenne du Soleil   (L)   : ",dansLimite360Degres(L))
M = j.anomalieMoyenne(T)
# ~ M = dansLimite360Degres(M)
print("Anomalie moyenne du Soleil    (M)   : ",dansLimite360Degres(M))
# ~ print("Anomalie moyenne du Soleil    (M)   : ",M)
e = j.excentricite(T)
print("Excentricité orbite terrestre (e)   : ",e)
C = j.equationDuCentre(T, M)
print("Équation au centre du Soleil  (C)   : ",C)
rond = j.longitudeVraie(L, C)
print("Longitude vraie du Soleil     (⊙)   : ",dansLimite360Degres(rond))
v = M + C
print("Anomalie vraie du Soleil      (v)   : ",dansLimite360Degres(v))
R = j.rayonVecteur(e, v)
print("Rayon vecteur du Soleil       (R)   : ",R)
delta = j.longitudeApparente(T, rond)
print("Longitude apparente           (λ)   : ",dansLimite360Degres(delta))
mob = j.meanObliquityOfEcliptic(sexToDecimal('23:26:21.448', False), T)
print("Obliquité moyenne             (ϵ)   : ",mob)
X, Y, Z = j.coordonneesRect(R, rond, mob)
print("Coordonnées,..                (X)   : ",X)
print("rectangulaires, équatoriales  (Y)   : ",Y)
print("géocentriques                 (Z)   : ",Z)


